from roughviz.charts.base import BaseChart

DEFAULT_COLORS = [
    "coral",
    "skyblue",
    "#66c2a5",
    "tan",
    "#8da0cb",
    "#e78ac3",
    "#a6d854",
    "#ffd92f",
    "tan",
    "orange",
]
DEFAULT_MARGIN = {"top": 50, "right": 20, "bottom": 70, "left": 100}


class Pie(BaseChart):
    CHART_TYPE = "Pie"

    PIE_KWARGS = {
        "font": "font",
        "highlight": "highlight",
        "inner_stroke_width": "innerStrokeWidth",
        "margin": "margin",
        "colors": "colors",
        "simplification": "simplification",
    }

    def __init__(
        self,
        data,
        values=None,
        labels=None,
        font=0,
        highlight="green",
        inner_stroke_width=0,
        margin=None,
        colors=None,
        simplification=0.2,
        **kwargs
    ):
        super().__init__(data, values, labels)

        self.opts["colors"] = colors or DEFAULT_COLORS
        self.opts["margin"] = margin or DEFAULT_MARGIN

        self.opts["font"] = font

        self.opts["highlight"] = highlight
        self.opts["innerStrokeWidth"] = inner_stroke_width
        self.opts["simplification"] = simplification

        self._set_kwargs(**kwargs)

    def set_options(self, **kwargs):
        mapping = {**self.BASE_KWARGS, **self.PIE_KWARGS}
        self._set_kwargs(mapping, **kwargs)



import pytest

from roughviz.charts.base import BaseChart


@pytest.mark.parametrize(
    "input_data, error, match_message",
    [
        (
            {"labels": ["a"], "value": [1]},
            ValueError,
            "must provide both values and labels as keys",
        ),
        ([1, 2, 3], TypeError, "Only valid type of data is str and dictionary"),
        ("wrong_data.txt", ValueError, "Wrong type of data"),
    ],
)
def test_invalid_input_data(input_data, error, match_message):
    with pytest.raises(error, match=match_message):
        BaseChart(data=input_data)


@pytest.mark.parametrize(
    "size, expected", [(0.5, "0.5rem"), (3, "3rem"), ("3rem", "3rem")]
)
def test_fontsize_converter(size, expected):
    assert BaseChart._fontsize_converter(size) == expected


@pytest.mark.parametrize("x, expected", [("a", "a"), ("", ""), (None, "")])
def test_xstr(x, expected):
    assert BaseChart._xstr(x) == expected



from roughviz.charts.base import BaseChart


class Barh(BaseChart):
    CHART_TYPE = "BarH"

    BARH_KWARGS = {
        "xlabel": "xLabel",
        "ylabel": "yLabel",
        "axis_fontsize": "axisFontSize",
        "label_fontsize": "labelFontSize",
        "axis_roughness": "axisRoughness",
        "axis_stroke_width": "axisStrokeWidth",
        "highlight": "highlight",
        "inner_stroke_width": "innerStrokeWidth",
        "padding": "padding",
        "stroke": "stroke",
        "simplification": "simplification",
    }

    def __init__(
        self,
        data,
        values=None,
        labels=None,
        xlabel=None,
        ylabel=None,
        axis_fontsize=1.5,
        label_fontsize=2,
        axis_roughness=0.5,
        highlight="coral",
        inner_stroke_width=0,
        padding=0.1,
        stroke="black",
        simplification=0.2,
        **kwargs
    ):
        super().__init__(data, values, labels)

        self.opts["xLabel"] = self._xstr(xlabel)
        self.opts["yLabel"] = self._xstr(ylabel)

        self.opts["axisFontSize"] = axis_fontsize
        self.opts["labelFontSize"] = label_fontsize

        self.opts["axisRoughness"] = axis_roughness
        self.opts["highlight"] = highlight
        self.opts["innerStrokeWidth"] = inner_stroke_width
        self.opts["padding"] = padding
        self.opts["simplification"] = simplification
        self.opts["stroke"] = stroke

        self._set_kwargs(**kwargs)

    def set_options(self, **kwargs):
        mapping = {**self.BASE_KWARGS, **self.BARH_KWARGS}
        self._set_kwargs(mapping, **kwargs)



from roughviz.charts.base import BaseChart

DEFAULT_COLORS = [
    "coral",
    "skyblue",
    "#66c2a5",
    "tan",
    "#8da0cb",
    "#e78ac3",
    "#a6d854",
    "#ffd92f",
    "tan",
    "orange",
]
DEFAULT_MARGIN = {"top": 50, "right": 20, "bottom": 70, "left": 100}
DATA_TYPE = (".csv", ".tsv")
SCATTER_DATA_KEYS = {"x", "y"}


class Scatter(BaseChart):
    CHART_TYPE = "Scatter"
    SCATTER_KWARGS = {
        "xlabel": "xLabel",
        "ylabel": "yLabel",
        "axis_fontsize": "axisFontSize",
        "label_fontsize": "labelFontSize",
        "axis_roughness": "axisRoughness",
        "axis_stroke_width": "axisStrokeWidth",
        "inner_stroke_width": "innerStrokeWidth",
        "padding": "padding",
        "highlight": "highlight",
        "stroke": "stroke",
        "radius": "radius",
        "simplification": "simplification",
    }

    def __init__(
        self,
        data,
        x=None,
        y=None,
        xlabel=None,
        ylabel=None,
        axis_fontsize=1.5,
        label_fontsize=2,
        axis_roughness=0.5,
        colors=None,
        color_zero=False,
        margin=None,
        font="Gaegu",
        highlight="green",
        inner_stroke_width=0,
        stroke="black",
        simplification=0.2,
        radius=8,
        **kwargs
    ):
        super().__init__(data, x, y)
        self._assign_input_values(x, y)

        self.opts["colors"] = colors or DEFAULT_COLORS
        self.opts["margin"] = margin or DEFAULT_MARGIN

        self.opts["xLabel"] = self._xstr(xlabel)
        self.opts["yLabel"] = self._xstr(ylabel)

        self.opts["axisFontSize"] = axis_fontsize
        self.opts["labelFontSize"] = label_fontsize
        self.opts["colorZero"] = color_zero
        self.opts["font"] = font
        self.opts["radius"] = radius
        self.opts["axisRoughness"] = axis_roughness
        self.opts["highlight"] = highlight
        self.opts["innerStrokeWidth"] = inner_stroke_width
        self.opts["simplification"] = simplification
        self.opts["stroke"] = stroke
        self._set_kwargs(**kwargs)

    @staticmethod
    def _check_input_data(data):
        if not isinstance(data, (str, dict)):
            raise TypeError("Only valid type of data is str and dictionary.")
        elif isinstance(data, str) and not data.endswith(DATA_TYPE):
            raise ValueError("Wrong type of data")
        elif isinstance(data, dict) and set(data.keys()) != SCATTER_DATA_KEYS:
            raise ValueError("Has to be x and y")

    def _assign_input_values(self, x, y):
        if isinstance(self.data, str) and (not x or not y):
            raise ValueError("You need to specify x and y as separate" "attributes.")

        self.opts["x"] = x
        self.opts["y"] = y

    def set_options(self, **kwargs):

        mapping = {**self.BASE_KWARGS, **self.SCATTER_KWARGS}
        self._set_kwargs(mapping, **kwargs)



from roughviz.charts.base import BaseChart

DEFAULT_COLORS = [
    "coral",
    "skyblue",
    "#66c2a5",
    "tan",
    "#8da0cb",
    "#e78ac3",
    "#a6d854",
    "#ffd92f",
    "tan",
    "orange",
]
DEFAULT_MARGIN = {"top": 50, "right": 20, "bottom": 70, "left": 100}
DATA_TYPE = (".csv", ".tsv")


class Line(BaseChart):
    CHART_TYPE = "Line"
    LINE_KWARGS = {
        "xlabel": "xLabel",
        "ylabel": "yLabel",
        "axis_fontsize": "axisFontSize",
        "label_fontsize": "labelFontSize",
        "axis_roughness": "axisRoughness",
        "axis_stroke_width": "axisStrokeWidth",
        "inner_stroke_width": "innerStrokeWidth",
        "padding": "padding",
        "highlight": "highlight",
        "colors": "colors",
        "stroke": "stroke",
        "margin": "margin",
        "simplification": "simplification",
        "legend": "legend",
        "lenged_position": "legendPosition",
        "circle": "circle",
        "circle_radius": "circleRadius",
        "circle_roughness": "circleRoughness",
    }

    def __init__(
        self,
        data,
        xlabel=None,
        ylabel=None,
        axis_fontsize=1,
        label_fontsize=2,
        axis_roughness=0.5,
        colors=None,
        margin=None,
        font="Gaegu",
        highlight="green",
        inner_stroke_width=0,
        stroke="black",
        simplification=0.2,
        legend=True,
        legend_position="right",
        circle=True,
        circle_radius=10,
        circle_roughness=2,
        **kwargs
    ):
        super().__init__(data)

        self.opts["colors"] = colors or DEFAULT_COLORS
        self.opts["margin"] = margin or DEFAULT_MARGIN

        # Line plot is slightly different than other plots, therefore, preprocessing
        # is needed beforehand
        ys = [kw for kw in kwargs if kw.startswith("y") and kw != "ylabel"]
        ys_dict = {y: kwargs.pop(y) for y in ys}

        self._assign_input_values(None, None, **ys_dict)
        self.opts["data"] = data

        self.opts["xLabel"] = self._xstr(xlabel)
        self.opts["yLabel"] = self._xstr(ylabel)

        self.opts["axisFontSize"] = axis_fontsize
        self.opts["labelFontSize"] = label_fontsize
        self.opts["font"] = font
        self.opts["axisRoughness"] = axis_roughness
        self.opts["highlight"] = highlight
        self.opts["innerStrokeWidth"] = inner_stroke_width
        self.opts["simplification"] = simplification
        self.opts["stroke"] = stroke
        self.opts["legend"] = legend
        self.opts["legendPosition"] = legend_position
        self.opts["circle"] = circle
        self.opts["circleRoughness"] = circle_roughness
        self.opts["circleRadius"] = circle_radius

        self._set_kwargs(**kwargs)

        # The fillStyle seems should not be assigned
        if "fillStyle" in self.opts:
            self.opts.pop("fillStyle")

    @staticmethod
    def _check_input_data(data):
        if not isinstance(data, str):
            raise TypeError("Only valid type of data is objects")
        elif isinstance(data, str) and not data.endswith(DATA_TYPE):
            raise ValueError(
                "Wrong format of data, has be to either csv or tsv format."
            )

    def _assign_input_values(self, values, labels, **ys_dict):
        for y, value in ys_dict.items():
            self.opts[y] = value

    def set_options(self, **kwargs):

        mapping = {**self.BASE_KWARGS, **self.LINE_KWARGS}
        self._set_kwargs(mapping, **kwargs)



from setuptools import setup, find_namespace_packages

__keywords__ = ["py-roughviz", "visualization", "sketchy charts", "hand-drawn charts"]

setup(
    name="py-roughviz",
    description="A Python implementation of JavaScript Library RoughViz to create sketchy charts.",
    version="0.1.0",
    author="Kaiqi Dong",
    author_email="kaiqidong1991@gmail.com",
    licnese="MIT",
    keywords=__keywords__,
    packages=find_namespace_packages(include=["py-roughviz.*"]),
    install_requires=["jinja2", "ipython"],
    download_url="https://github.com/charlesdong1991/py-roughviz/archive/v0.1.1.tar.gz",
    tests_require=["pytest"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries",
    ],
)



import json
import uuid
from numbers import Number
from typing import Optional, Union

from roughviz.render.engine import RenderEngine


DATA_TYPE = (".csv", ".tsv")
DATA_KEYS = {"labels", "values"}


class BaseChart(RenderEngine):
    """
    This is the base class where all charts are inherited from
    """
    BASE_KWARGS = {
        "title": "title",
        "interactive": "interactive",
        "bowing": "bowing",
        "fill_style": "fillStyle",
        "fill_weight": "fillWeight",
        "stroke_width": "strokeWidth",
        "roughtness": "roughness",
        "tooltip_fontsize": "tooltipFontSize",
        "title_fontsize": "titleFontSize",
        "width": "width",
        "height": "height",
    }

    def __init__(
        self,
        data,
        values: Optional[str] = None,
        labels: Optional[str] = None,
        title: Optional[str] = None,
        title_fontsize: Optional[Union[int, float]] = 0.95,
        width: Optional[int] = 800,
        height: Optional[int] = 600,
        interactive: bool = True,
        bowing: Optional[float] = 0.2,
        fill_style: Optional[str] = "cross-hatch",
        fill_weight: Optional[Union[int, float]] = 0,
        stroke_width: Optional[Union[int, float]] = 1,
        roughness: Optional[Union[int, float]] = 1,
        tooltip_fontsize: Optional[Union[int, float]] = 0.95,
        **kwargs,
    ):
        super().__init__()
        self.opts: dict = {}

        self.data = data
        self._check_input_data(data)

        self._assign_input_values(labels, values)
        self.opts["data"] = data

        self.opts["title"] = self._xstr(title)
        self.opts["width"] = width
        self.opts["height"] = height
        self.opts["interactive"] = interactive
        self.opts["bowing"] = bowing
        self.opts["fillStyle"] = fill_style
        self.opts["fillWeight"] = fill_weight
        self.opts["strokeWidth"] = stroke_width
        self.opts["roughness"] = roughness
        self.opts["tooltipFontSize"] = tooltip_fontsize
        self.opts["titleFontSize"] = title_fontsize

    def render_to_tmpl(self):
        """This method is used to render the content to the template
        """
        # Assign an id for container.
        self.element_id = uuid.uuid4().hex
        self.opts["element"] = "#" + self.element_id

        self._addition_conversion()
        self.json_content = json.dumps(self.opts)

    @staticmethod
    def _check_input_data(data):
        """Check the input data type and format.
        """
        if not isinstance(data, (str, dict)):
            raise TypeError("Only valid type of data is str and dictionary.")
        elif isinstance(data, str) and not data.endswith(DATA_TYPE):
            raise ValueError("Wrong type of data")
        elif isinstance(data, dict) and set(data.keys()) != DATA_KEYS:
            raise ValueError(
                "If input data is dictionary, you must provide both values and labels as keys"
            )

    def _assign_input_values(self, labels, values):
        """Assign the values for input data.
        """
        if isinstance(self.data, str) and (not labels or not values):
            raise ValueError(
                "You need to specify labels and values as separate" "attributes."
            )
        self.opts["labels"] = labels
        self.opts["values"] = values

    def _addition_conversion(self):
        self._convert_fontsize_unit()

    def _convert_fontsize_unit(self):
        fontsize_opts = [opt for opt in self.opts if opt.endswith("FontSize")]

        for opt in fontsize_opts:
            self.opts[opt] = self._fontsize_converter(self.opts[opt])

    @staticmethod
    def _xstr(s):
        return "" if s is None else s

    @staticmethod
    def _fontsize_converter(size):
        """Convert the fontsize if it is numeric. Since the unit of fontsize in roughviz is `rem`."""
        if isinstance(size, Number):
            return str(size) + "rem"
        return size

    def set_title(self, title: str, fontsize: Optional[Union[int, float]] = None):
        """Set the title for plots.

        Args:
            title: the title of the plot
            fontsize: the fontsize of the title, if None, use the default size.
        """
        self.opts["title"] = self._xstr(title)
        if fontsize:
            self.opts["titleFontSize"] = fontsize
        return self

    def set_xlabel(self, xlabel: str, fontsize: Optional[Union[int, float]] = None):
        """Set the xlabel for plots.

        Args:
            xlabel: the xlabel of the plot
            fontsize: the fontsize of the xlabel, if None, use the default size.
        """
        self.opts["xLabel"] = self._xstr(xlabel)
        if fontsize:
            self.opts["labelFontSize"] = fontsize
        return self

    def set_ylabel(self, ylabel: str, fontsize: Optional[Union[int, float]] = None):
        """Set the ylabel for plots.

        Args:
            ylabel: the ylabel of the plot
            fontsize: the fontsize of the ylabel, if None, use the default size.
        """
        self.opts["yLabel"] = self._xstr(ylabel)
        if fontsize:
            self.opts["labelFontSize"] = fontsize
        return self

    def set_figsize(self, figsize: tuple):
        """Set the figsize for plots, in a format of (width, height).

        Args:
            figsize: the figsize of the plot
        """
        if not isinstance(figsize, tuple):
            raise ValueError("figsize has to be a tuple.")
        self.opts["width"] = figsize[0]
        self.opts["height"] = figsize[1]

        return self

    def set_legend(self, legend: bool = True, legend_position: Optional[str] = "right"):
        """Set the legend for plots.

        Args:
            legend: if having legend or not, default is True.
            legend_position: the fontsize of the title, default is right.
        """
        if legend:
            self.opts["legendPosition"] = legend_position
        else:
            self.opts["legend"] = legend
        return self

    def _set_kwargs(self, mapping: Optional[dict] = None, **kwargs):
        """Making a mapping for keyword arguments due to syntax difference and assign corresponding
        values to mapped keywords.
        """
        if mapping is None:
            mapping = self.BASE_KWARGS
        for kw, kw_value in kwargs.items():
            self.opts[mapping[kw]] = kw_value



import os

from jinja2 import Template
from IPython.core.display import display, HTML

ABS_PATH = os.path.abspath(os.path.dirname(__file__))


class RenderEngine:
    def __init__(self, tmpl_file: str = "templates/jupyter_notebook.html"):
        tmpl_file = os.path.join(ABS_PATH, tmpl_file)
        self.tmpl_file = tmpl_file

    def render_notebook(self):
        if hasattr(self, "render_to_tmpl"):
            self.render_to_tmpl()

        with open(self.tmpl_file) as f:
            tmpl_file = f.read()
        tmpl = Template(tmpl_file)

        output = tmpl.render(chart=self)
        return display(HTML(output))

    show = render_notebook



from roughviz.charts.base import BaseChart


class Donut(BaseChart):
    CHART_TYPE = "Donut"

    DONUT_KWARGS = {
        "font": "font",
        "highlight": "highlight",
        "inner_stroke_width": "innerStrokeWidth",
        "margin": "margin",
        "simplification": "simplification",
    }

    def __init__(
        self,
        data,
        values=None,
        labels=None,
        font=0,
        highlight="green",
        inner_stroke_width=0,
        simplification=0.2,
        **kwargs
    ):
        super().__init__(data, values, labels)

        self.opts["font"] = font
        self.opts["highlight"] = highlight
        self.opts["innerStrokeWidth"] = inner_stroke_width
        self.opts["simplification"] = simplification

        self._set_kwargs(**kwargs)

    def set_options(self, **kwargs):
        mapping = {**self.BASE_KWARGS, **self.DONUT_KWARGS}
        self._set_kwargs(mapping, **kwargs)
